DAT fileName  byte "Servo120129j"  ' I like to have the name of the program
                                   ' displayed in a terminal window.
CON{{

  111027a start working on a sevo demo.
  27d modify equations.
  27e keep servo #15 centered.
  27f increase _FramesPerPeriod (slow down servos).
  27g add displayPosition array so frame numbers match position
  values.
  27h fix byte sized frame problem.
  27i change format so code fits forum code box.
  120120a Big Change.  Make each servo have different
  end points and periods.
  Increase numbers of servos to 28.
  120129j use separate cog to change phase and other
  parameters.
}}

CON

  _Clkmode = xtal1 + pll16x
  _Xinfreq = 5_000_000

  _DebugBaud = 9600
 
  _DefaultMaxPulse = 2000
  _DefaultMinPulse = 1000
  _AbsoluteMaxPulse = 3000
  _AbsoluteMinPulse = 300
  _Stop = 1500
  _Center = _Stop
  _ServosInUse = 32 '28        
  _DefaultFramesPerPeriod = 300
  _MaxPeriod = 1000
  _MinPeriod = 50
  _PositionsPerRow = 4
  _FullCircle = 36000

  _DelaysMonitored = 7
  
VAR

  long time, timeOld, frameInterval, nextFrameTime, debugStack[100]
  
  long servoPosition[_ServosInUse], frameDelay[_DelaysMonitored], maxPosition[_ServosInUse], minPosition[_ServosInUse]
  long displayPosition[_ServosInUse], displayDelay[_DelaysMonitored], frameCount[_ServosInUse], frameCountOld
  long period[_ServosInUse], tempTarget[_ServosInUse], longestTask  
  
  
  byte servoPin[_ServosInUse]
   

OBJ

  Debug : "Simple_Serial"
  Servo : "Servo32v7"

  'Debug : "Parallax Serial Terminal"
  
PUB Setup | localIndex, t1, t2

  'Debug.start(57600)
  waitcnt(clkfreq * 3 + cnt)
  
  repeat localIndex from 0 to _ServosInUse - 1   ' Set pin numbers.  They don't need to be consecutive.
    servoPin[localIndex] := localIndex
    maxPosition[localIndex] := _DefaultMaxPulse + 500 '+ ((localIndex * (_AbsoluteMaxPulse - _DefaultMaxPulse)) / _ServosInUse)
    minPosition[localIndex] := _DefaultMinPulse - 500 '- ((localIndex * (_DefaultMinPulse - _AbsoluteMinPulse)) / _ServosInUse)
    period[localIndex] := _DefaultFramesPerPeriod / 2 '_MinPeriod + ((localIndex * (_MaxPeriod - _MinPeriod)) / _ServosInUse)
    servoPosition[localIndex] := _Center
    frameCount[localIndex] := (period[localIndex] / 2) - 1
   { Debug.str(string(13, "servoPin["))
    Dec(localIndex)
    Debug.str(string("] = "))
    Dec(servoPin[localIndex])
    Debug.str(string(", maxPosition = "))
    Dec(maxPosition[localIndex])
    Debug.str(string(", minPosition = "))
    Dec(minPosition[localIndex])
    Debug.str(string(", period = "))
    Dec(period[localIndex])
    Debug.str(string(", frameCount = "))
    Dec(frameCount[localIndex]) }

    
  frameInterval := clkfreq / 50

  
  'cognew(DebugLoop, @debugStack)                 ' starts a cog 6 left
  waitcnt(clkfreq / 20 + cnt)                           
  
  Servo.Start                                    ' starts a cog 5 left
 
  waitcnt(clkfreq / 20 + cnt)
  'SetEndPointLow(localFirstIndex, localFirstEnd, localLastIndex, localLastEnd)
  'SetEndPointHigh(localFirstIndex, localFirstEnd, localLastIndex, localLastEnd)
  nextFrameTime := cnt + frameInterval
  longestTask := 0
  repeat
    ' start with all servos in same position, same period and same end points
    SetPeriod(0, _DefaultFramesPerPeriod, _ServosInUse - 1, _DefaultFramesPerPeriod)
    SetEndPointLow(0, _DefaultMinPulse, _ServosInUse - 1, _DefaultMinPulse) 
    SetEndPointHigh(0, _DefaultMaxPulse, _ServosInUse - 1, _DefaultMaxPulse)
    SetPhase(0, _FullCircle / 4, _ServosInUse - 1, _FullCircle / 4)
    {
    ' oscillate linear algorthm
    frameDelay[0] := cnt - (nextFrameTime - frameInterval)
    if frameDelay[0] > longestTask
      longestTask := frameDelay[0]
    repeat 25
      waitcnt(nextFrameTime)
      nextFrameTime += frameInterval
      Servo.Set(0, _Center)
      repeat result from 1 to _ServosInUse - 1
        Servo.Set(result, 2000)
    }    
    repeat _DefaultFramesPerPeriod * 2               
      OscillateLinear(0, _ServosInUse - 1)
    {repeat 25
      waitcnt(nextFrameTime)
      nextFrameTime += frameInterval
      repeat result from 0 to 0
        Servo.Set(result, _Center)
      Servo.Set(1, _Center)
      repeat result from 2 to _ServosInUse - 1
        Servo.Set(result, _Center)
    }
    ' oscillate pseudosinusoidal  
    repeat _DefaultFramesPerPeriod * 2               
      OscillateSinusoidal(0, _ServosInUse - 1)
    { repeat 25
      waitcnt(nextFrameTime)
      nextFrameTime += frameInterval
      repeat result from 0 to 1
        Servo.Set(result, 2000)
      Servo.Set(2, _Center)
      repeat result from 3 to _ServosInUse - 1
        Servo.Set(result, 2000)
    
    ' move end points, high number servo move more, and repeat oscillations  
    SetEndPointLow(0, _DefaultMinPulse + 250, _ServosInUse - 1, _AbsoluteMinPulse) 
    SetEndPointHigh(0, _DefaultMaxPulse - 250, _ServosInUse - 1, _AbsoluteMaxPulse)
    frameDelay[1] := cnt - (nextFrameTime - frameInterval)
    if frameDelay[1] > longestTask
      longestTask := frameDelay[1]
    repeat _DefaultFramesPerPeriod * 2               
      OscillateLinear(0, _ServosInUse - 1)
    t1 := 3
    repeat 25
      waitcnt(nextFrameTime)
      nextFrameTime += frameInterval
      repeat result from 0 to t1 - 1
        Servo.Set(result, 2000)
      Servo.Set(t1, _Center)
      repeat result from t1 + 1 to _ServosInUse - 1
        Servo.Set(result, 2000)
    repeat _DefaultFramesPerPeriod * 2               
      OscillateSinusoidal(0, _ServosInUse - 1)

    t1 := 3
    repeat 25
      waitcnt(nextFrameTime)
      nextFrameTime += frameInterval
      repeat result from 0 to t1 - 1
        Servo.Set(result, 2000)
      Servo.Set(t1, _Center)
      repeat result from t1 + 1 to _ServosInUse - 1
        Servo.Set(result, 2000)
    
    ' set all ccw
    repeat _DefaultFramesPerPeriod / 2
      SetAlternating(_DefaultMaxPulse, _DefaultMaxPulse)

    'set all cw  
    repeat _DefaultFramesPerPeriod / 2
      SetAlternating(_DefaultMinPulse, _DefaultMinPulse)

    'flip back and forth alternate servos  
    repeat _DefaultFramesPerPeriod
      SetAlternating(_DefaultMaxPulse, _DefaultMinPulse)
    repeat _DefaultFramesPerPeriod
      SetAlternating(_DefaultMinPulse, _DefaultMaxPulse)
    repeat _DefaultFramesPerPeriod
      SetAlternating(_DefaultMaxPulse, _DefaultMinPulse)
    repeat _DefaultFramesPerPeriod
      SetAlternating(_DefaultMinPulse, _DefaultMaxPulse)
    repeat _DefaultFramesPerPeriod
      SetAlternating(_DefaultMaxPulse, _DefaultMinPulse)
    repeat _DefaultFramesPerPeriod
      SetAlternating(_DefaultMinPulse, _DefaultMaxPulse)
    
    ' change periods, higher number servos will move faster, but endpoints will be the same
    SetPeriod(0, _DefaultFramesPerPeriod, _ServosInUse - 1, _DefaultFramesPerPeriod / 8)
    SetEndPointLow(0, _DefaultMinPulse, _ServosInUse - 1, _DefaultMinPulse) 
    SetEndPointHigh(0, _DefaultMaxPulse, _ServosInUse - 1, _DefaultMaxPulse)
    frameDelay[2] := cnt - (nextFrameTime - frameInterval)
    if frameDelay[2] > longestTask
      longestTask := frameDelay[2]
    repeat _DefaultFramesPerPeriod * 2               
      OscillateLinear(0, _ServosInUse - 1)
    repeat _DefaultFramesPerPeriod * 2               
      OscillateSinusoidal(0, _ServosInUse - 1)

    ' signal change by having all servos centered  
    repeat _DefaultFramesPerPeriod / 2
      SetAlternating(_Center, _Center)

    ' spread end points out again  and oscillate at a variety of periods   
    SetEndPointLow(0, _DefaultMinPulse, _ServosInUse - 1, _AbsoluteMinPulse) 
    SetEndPointHigh(0, _DefaultMaxPulse, _ServosInUse - 1, _AbsoluteMaxPulse)
    frameDelay[3] := cnt - (nextFrameTime - frameInterval)
    if frameDelay[3] > longestTask
      longestTask := frameDelay[3]
    repeat _DefaultFramesPerPeriod * 2               
      OscillateLinear(0, _ServosInUse - 1)
    repeat _DefaultFramesPerPeriod * 2               
      OscillateSinusoidal(0, _ServosInUse - 1)
    }
    ' endpoints and periods will all be the same, just phase will be different
    'SetPeriod(0, _DefaultFramesPerPeriod, _ServosInUse - 1, _DefaultFramesPerPeriod)
    'SetEndPointLow(0, _DefaultMinPulse, _ServosInUse - 1, _DefaultMinPulse) 
    'SetEndPointHigh(0, _DefaultMaxPulse, _ServosInUse - 1, _DefaultMaxPulse)
    SetEndPointLow(0, _DefaultMinPulse - 100, 15, _DefaultMinPulse - 100) 
    SetEndPointHigh(0, _DefaultMaxPulse + 100, 15, _DefaultMaxPulse + 100)
    SetEndPointLow(16, _DefaultMinPulse - 250, _ServosInUse - 1, _DefaultMinPulse - 250) 
    SetEndPointHigh(16, _DefaultMaxPulse + 25, _ServosInUse - 1, _DefaultMaxPulse + 250)
    SetPhase(0, 0, _ServosInUse - 1, _FullCircle)
    frameDelay[4] := cnt - (nextFrameTime - frameInterval)
    if frameDelay[4] > longestTask
      longestTask := frameDelay[4]
    'repeat _DefaultFramesPerPeriod * 2               
    '  OscillateLinear(0, _ServosInUse - 1)
    repeat _DefaultFramesPerPeriod * 4              
      OscillateSinusoidal(0, _ServosInUse - 1)

    ' move end points out
    {SetEndPointLow(0, _DefaultMinPulse, _ServosInUse - 1, _AbsoluteMinPulse) 
    SetEndPointHigh(0, _DefaultMaxPulse, _ServosInUse - 1, _AbsoluteMaxPulse)
    frameDelay[5] := cnt - (nextFrameTime - frameInterval)
    if frameDelay[5] > longestTask
      longestTask := frameDelay[5]
    repeat _DefaultFramesPerPeriod * 2               
      OscillateLinear(0, _ServosInUse - 1)
    repeat _DefaultFramesPerPeriod * 2               
      OscillateSinusoidal(0, _ServosInUse - 1)
    repeat _DefaultFramesPerPeriod / 2
      SetAlternating(_Center, _Center) 
    t1 := 8
    repeat 25
      waitcnt(nextFrameTime)
      nextFrameTime += frameInterval
      repeat result from 0 to t1 - 1
        Servo.Set(result, 2000)
      Servo.Set(t1, _Center)
      repeat result from t1 + 1 to _ServosInUse - 1
        Servo.Set(result, 2000)        
    SetEndPointLow(0, _DefaultMinPulse - 100, 15, _DefaultMinPulse - 100) 
    SetEndPointHigh(0, _DefaultMaxPulse + 100, 15, _DefaultMaxPulse + 100)
    SetEndPointLow(16, _DefaultMinPulse - 250, _ServosInUse - 1, _DefaultMinPulse - 250) 
    SetEndPointHigh(16, _DefaultMaxPulse + 25, _ServosInUse - 1, _DefaultMaxPulse + 250)
    }
    SetPeriod(0, _DefaultFramesPerPeriod, 7, _DefaultFramesPerPeriod)
    SetPeriod(8, _DefaultFramesPerPeriod / 2, 15, _DefaultFramesPerPeriod / 2)
    SetPeriod(16, _DefaultFramesPerPeriod / 4, _ServosInUse - 1, _DefaultFramesPerPeriod / 4)
    'SetPhase(0, 0, 15, _FullCircle)
    'SetPhase(16, 0, 15, _FullCircle * 2)
    frameDelay[6] := cnt - (nextFrameTime - frameInterval)
    if frameDelay[6] > longestTask
      longestTask := frameDelay[6]
    'repeat _DefaultFramesPerPeriod * 2               
    '  OscillateLinear(0, _ServosInUse - 1)
    'repeat _DefaultFramesPerPeriod * 2               
    '  OscillateLinear(0, _ServosInUse - 1)

    repeat _DefaultFramesPerPeriod * 8               
      OscillateSinusoidal(0, _ServosInUse - 1)
    'repeat _DefaultFramesPerPeriod / 2
    '  SetAlternating(_Center, _Center)
    SetPeriod(0, _DefaultFramesPerPeriod, _ServosInUse - 1, _DefaultFramesPerPeriod / 16)
    repeat _DefaultFramesPerPeriod * 8               
      OscillateSinusoidal(0, _ServosInUse - 1)
      
PUB OscillateLinear(localFirstIndex, localLastIndex) | localIndex
              
 waitcnt(nextFrameTime)
 nextFrameTime += frameInterval
{ Debug.str(string(13, 13, " Servo Positions"))    
 
 repeat localIndex from 0 to _ServosInUse - 1      
   if (localIndex // _PositionsPerRow) == 0
     Debug.char(13)
     Debug.str(string("SP["))  
   else
     Debug.str(string(", SP["))  
   Dec(localIndex)
   Debug.str(string("] = "))  }
   'Dec(frameCount[localIndex]) 
   'Debug.str(string(", ")) 
   'Dec(servoPosition[localIndex])
 repeat localIndex from 0 to _ServosInUse - 1
   if ++frameCount[localIndex] => period[localIndex]
     frameCount[localIndex] := 0
   {Debug.char(13)
   Debug.str(string("frameCount["))
   Dec(localIndex)
   Debug.str(string("] = "))
   Dec(frameCount[localIndex])
   Debug.str(string(", maxPosition["))
   Dec(localIndex)
   Debug.str(string("] = "))
   Dec(maxPosition[localIndex])
   Debug.str(string("period["))
   Dec(localIndex)
   Debug.str(string("] = "))
   Dec(period[localIndex])
   Debug.str(string(", minPosition["))
   Dec(localIndex)
   Debug.str(string("] = "))
   Dec(minPosition[localIndex])  }
   if frameCount[localIndex] > period[localIndex] / 2 
     servoPosition[localIndex] := maxPosition[localIndex] - ((2 * (frameCount[localIndex] - (period[localIndex] / 2)) * (maxPosition[localIndex] - minPosition[localIndex]))/ period[localIndex])
     {Debug.str(string(13, "2 * (frameCount[localIndex] - (period[localIndex] / 2)) = "))
     Dec(2 * (frameCount[localIndex] - (period[localIndex] / 2)))
      }
   else
     servoPosition[localIndex] := minPosition[localIndex] + ((2 * frameCount[localIndex] * (maxPosition[localIndex] - minPosition[localIndex]))/ period[localIndex])
     {Debug.str(string(13, "2 * frameCount[localIndex] = "))
     Dec(2 * frameCount[localIndex])
   Debug.str(string(13, "(maxPosition[localIndex] - minPosition[localIndex]) = "))
   Dec((maxPosition[localIndex] - minPosition[localIndex]))
   Debug.str(string(13, "servoPosition[localIndex] = "))
   Dec(servoPosition[localIndex])
   }
   Servo.Set(servoPin[localIndex], servoPosition[localIndex])
     
PUB OscillateSinusoidal(localFirstIndex, localLastIndex) | localIndex, pseudoPeriod, pseudoRange, pseudoTime

  waitcnt(nextFrameTime)
  nextFrameTime += frameInterval

  repeat localIndex from 0 to _ServosInUse - 1
    if ++frameCount[localIndex] => period[localIndex]
      frameCount[localIndex] := 0
   
    pseudoPeriod := (period[localIndex] * period[localIndex]) / 16  ' fourth period squared
    pseudoRange := (maxPosition[localIndex] - minPosition[localIndex]) / 2
    if frameCount[localIndex] > period[localIndex] / 2
      if frameCount[localIndex] > (period[localIndex] * 3) / 4
        pseudoTime := period[localIndex] - frameCount[localIndex]
        pseudoTime *= pseudoTime
        servoPosition[localIndex] := minPosition[localIndex] + ((pseudoTime * pseudoRange) / pseudoPeriod)
      else
        pseudoTime := frameCount[localIndex] - (period[localIndex] / 2)
        pseudoTime *= pseudoTime 
        servoPosition[localIndex] := maxPosition[localIndex] - ((pseudoTime * pseudoRange) / pseudoPeriod) 
     
    else
      if frameCount[localIndex] > period[localIndex] / 4
        pseudoTime := (period[localIndex] / 2) - frameCount[localIndex] 
        pseudoTime *= pseudoTime 
        servoPosition[localIndex] := maxPosition[localIndex] - ((pseudoTime * pseudoRange) / pseudoPeriod)
      else
        pseudoTime := frameCount[localIndex] * frameCount[localIndex]
        servoPosition[localIndex] := minPosition[localIndex] + ((pseudoTime * pseudoRange) / pseudoPeriod)
    Servo.Set(servoPin[localIndex], servoPosition[localIndex])

PUB SetAlternating(firstPosition, secondPosition) | localIndex
     
  waitcnt(nextFrameTime)
  nextFrameTime += frameInterval
    
  repeat localIndex from 0 to _ServosInUse - 1
    if ++frameCount[localIndex] => period[localIndex]
      frameCount[localIndex] := 0
    if localIndex // 2 == 0
      servoPosition[localIndex] := firstPosition
    else  
      servoPosition[localIndex] := secondPosition
    Servo.Set(servoPin[localIndex], servoPosition[localIndex])
       
PUB SetEndPointLow(localFirstIndex, localFirstEnd, localLastIndex, localLastEnd) | localIndex

  localFirstEnd <#= _AbsoluteMaxPulse
  localFirstEnd #>= _AbsoluteMinPulse
  localLastEnd <#= _AbsoluteMaxPulse
  localLastEnd #>= _AbsoluteMinPulse

  if localLastIndex < localFirstIndex ' method works with first pin lower than last so switch them if needed
    localIndex := localFirstEnd
    localFirstEnd := localLastEnd
    localLastEnd := localIndex
    localIndex := localFirstIndex
    localFirstIndex := localLastIndex
    localLastIndex := localIndex
    
  if localFirstEnd < localLastEnd
    repeat localIndex from 0 to localLastIndex - localFirstIndex  
      minPosition[localFirstIndex + localIndex] := localFirstEnd + ((localIndex * (localLastEnd - localFirstEnd)) / ((localLastIndex - localFirstIndex) + 1))
  elseif localFirstEnd > localLastEnd
    repeat localIndex from 0 to localLastIndex - localFirstIndex  
      minPosition[localFirstIndex + localIndex] := localLastEnd - ((localIndex * (localFirstEnd - localLastEnd)) / ((localLastIndex - localFirstIndex) + 1))
  else ' all end points are the same
    repeat localIndex from 0 to localLastIndex - localFirstIndex  
      minPosition[localFirstIndex + localIndex] := localFirstEnd
      
PUB SetEndPointHigh(localFirstIndex, localFirstEnd, localLastIndex, localLastEnd) | localIndex    

  localFirstEnd <#= _AbsoluteMaxPulse
  localFirstEnd #>= _AbsoluteMinPulse
  localLastEnd <#= _AbsoluteMaxPulse
  localLastEnd #>= _AbsoluteMinPulse
  
  if localLastIndex < localFirstIndex ' method works with first pin lower than last so switch them if needed
    localIndex := localFirstEnd
    localFirstEnd := localLastEnd
    localLastEnd := localIndex
    localIndex := localFirstIndex
    localFirstIndex := localLastIndex
    localLastIndex := localIndex
    
  if localFirstEnd < localLastEnd
    repeat localIndex from 0 to localLastIndex - localFirstIndex  
      maxPosition[localIndex] := localFirstEnd + ((localIndex * (localLastEnd - localFirstEnd)) / ((localLastIndex - localFirstIndex) + 1))
  elseif localFirstEnd > localLastEnd
    repeat localIndex from 0 to localLastIndex - localFirstIndex  
      maxPosition[localIndex] := localLastEnd - ((localIndex * (localFirstEnd - localLastEnd)) / ((localLastIndex - localFirstIndex) + 1))
  else ' all end points are the same
    repeat localIndex from 0 to localLastIndex - localFirstIndex  
      maxPosition[localIndex] := localFirstEnd
     
PUB SetPeriod(localFirstIndex, localFirstFrames, localLastIndex, localLastFrames) | localIndex

  localFirstFrames <#= _MaxPeriod
  localFirstFrames #>= _MinPeriod
  localLastframes <#= _MaxPeriod
  localLastframes #>= _MinPeriod

  if localLastIndex < localFirstIndex ' method works with first pin lower than last so switch them if needed
    localIndex := localFirstFrames
    localFirstFrames := localLastframes
    localLastframes := localIndex
    localIndex := localFirstIndex
    localFirstIndex := localLastIndex
    localLastIndex := localIndex
  
  if localFirstFrames < localLastFrames
    repeat localIndex from 0 to localLastIndex - localFirstIndex  
      period[localIndex] := localFirstFrames + ((localIndex * (localLastFrames - localFirstFrames)) / ((localLastIndex - localFirstIndex) + 1))
  elseif localFirstFrames > localLastFrames
    repeat localIndex from 0 to localLastIndex - localFirstIndex  
      period[localIndex] := localFirstFrames - ((localIndex * (localFirstFrames - localLastFrames)) / ((localLastIndex - localFirstIndex) + 1))
  else
    repeat localIndex from 0 to localLastIndex - localFirstIndex
      period[localIndex] := localFirstFrames
       
PUB SetPhase(localFirstIndex, localFirstPhase, localLastIndex, localLastPhase) | localIndex
'' Phase should be in units of degrees times 100.

  if localLastIndex < localFirstIndex ' method works with first pin lower than last so switch them if needed
    localIndex := localFirstPhase
    localFirstPhase := localLastPhase
    localLastPhase := localIndex
    localIndex := localFirstIndex
    localFirstIndex := localLastIndex
    localLastIndex := localIndex
  
  if localFirstPhase < localLastPhase
    repeat localIndex from 0 to localLastIndex - localFirstIndex  
      result := localFirstPhase + ((localIndex * (localLastPhase - localFirstPhase)) / ((localLastIndex - localFirstIndex) + 1))
      repeat while result => _FullCircle
        result -= _FullCircle
      frameCount[localIndex] := (result * period[localIndex]) / _FullCircle
  elseif localFirstPhase > localLastPhase
    repeat localIndex from 0 to localLastIndex - localFirstIndex  
      result := localFirstPhase - ((localIndex * (localFirstPhase - localLastPhase)) / ((localLastIndex - localFirstIndex) + 1))
      repeat while result => _FullCircle
        result -= _FullCircle
      frameCount[localIndex] := (result * period[localIndex]) / _FullCircle
  else
    repeat localIndex from 0 to localLastIndex - localFirstIndex
      result := localFirstPhase
      repeat while result => _FullCircle
        result -= _FullCircle
      frameCount[localIndex] := (result * period[localIndex]) / _FullCircle
       
PUB DebugLoop | localIndex
'' We'll use a seperate loop to send debug information
'' so it doesn't slow down the servo calculations.
'' It is likely that not all frames will be displayed
'' since the data can't be transmitted fast enough to
'' keep up with how quickly the seroPosition data is
'' changed.

  Debug.init(31, 30, _DebugBaud)
  Debug.str(string(13, "Debug Started "))
  Debug.str(string(13, "fileName = "))
  Debug.str(@fileName)
  Debug.tx(13)
  frameCountOld := period[0] + 1
  ' not a legitamate frame so frameCount is sure not to be equal to it.

  repeat
    repeat while frameCount[0] == frameCountOld
    frameCountOld := frameCount[0]
    longmove(@displayPosition, @servoPosition, _ServosInUse + _DelaysMonitored)
    ' move the servoPosition values so they don't change while they
    ' are being displayed.
    
    Debug.str(string(13, 13, " Frame # "))
    Dec(frameCountOld)
    Debug.str(string(" Servo Positions"))
    Debug.str(string(13, " longestTask = "))
    Dec(longestTask)
    'Debug.str(string(" Delays"))
    repeat localIndex from 0 to _DelaysMonitored - 1      
      if localIndex == 0
        Debug.tx(13)
        Debug.str(string(" delay["))  
      else
        Debug.str(string(", delay["))  
      Dec(localIndex)
      Debug.str(string("] = "))
      Dec(displayDelay[localIndex])
    'Debug.tx(13)
    
    repeat localIndex from 0 to _ServosInUse - 1      
      if (localIndex // _PositionsPerRow) == 0
        Debug.tx(13)
        Debug.str(string(" SP["))  
      else
        Debug.str(string(", SP["))  
      Dec(localIndex)
      Debug.str(string("] = "))
      Dec(displayPosition[localIndex])
      
PUB Dec(value) | i, x
'' Since Simple_Serial doesn't have a dec method.
'' Copied from FullDuplexSerial
'' Print a decimal number

  x := value == NEGX            'Check for max negative
  if value < 0
    value := ||(value+x)        'If negative, make positive; adjust for max negative
    Debug.tx("-")               'and output sign

  i := 1_000_000_000            'Initialize divisor

  repeat 10                     'Loop for 10 digits
    if value => i                                                               
      Debug.tx(value / i + "0" + x*(i == 1)) 'If non-zero digit, output digit; adjust for max negative
      value //= i               'and digit from value
      result~~                  'flag non-zero found
    elseif result or i == 1
      Debug.tx("0")             'If zero digit (or only digit) output it
    i /= 10                     'Update divisor
       